// -*- Mode: c++ -*-
// $Id: arcball.h,v 1.3 2000/07/19 19:08:15 jianbo Exp $
// $Source: /tools/cvs-repos/illustrate/include/arcball.h,v $

/* Subdivide V2.0
   Copyright (C) 2000 Henning Biermann, Denis Zorin, NYU

This file is part of Subdivide.

Subdivide is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Subdivide is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Subdivide; see the file COPYING.  If not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.  */

#ifndef __ARCBALL_H__
#define __ARCBALL_H__

#include "quat.h"
#include "hmatrix.h"

#define MAX_SET_SIZE 3 

//: Arcball rotation for UI
class ArcBall { 
public:

  enum AxisSet {NoAxes, WorldAxes, BodyAxes, NSets};

  enum AxesNames { X = 0,Y = 1 ,Z = 2};

  void Update();
  void Init();

  ArcBall() { Init(); }


  void SetRadius( double r ) { radius = r; } 
  double Radius() const { return radius; }
  void SetPosition( const cvec3d& c) { center = c; 
          combined = HMatrix::Translation(center)*mNow;}
  const cvec3d& Position() const { return center;  }

  void SetRotation( const HMatrix& rot ) { mNow = rot; 
          combined = HMatrix::Translation(center)*mNow;} 
  const HMatrix&  Rotation() { return mNow; }

  void PickingRay( const cvec3d& rs, const cvec3d& rd)
  { rayStartNow = rs; rayDirNow = rd; }
  
  void UseSet(AxisSet aset) { 
    if (!dragging) axisSet = aset; 
  }
  
  // appearance
  void SetInactiveColor(const cvec3f& color) { inactiveColor = color;  }

  void SetActiveColor  (const cvec3f& color) { activeColor = color;  }

  void SetDragColor    (const cvec3f& color) { dragColor = color;  }

  void SetOutlineColor (const cvec3f& color) { outlineColor = color;  }

  void SetResultColor  (const cvec3f& color) { resultColor = color;  }


  // Begin drawing arc for all drags combined
  void ShowResult() { showResult = GL_TRUE; }
  // Stop drawing arc for all drags combined. 
  void HideResult() { showResult = GL_FALSE; }

  void BeginDrag();
  void EndDrag();
  void Draw(const cvec3d& camerapos );
  void DrawCompleteBall(const cvec3d& camerapos );

 private:
  cvec3d RayOnSphere(const cvec3d& ray_start, 
		     const cvec3d& ray_dir);

  void DrawOuterRing(const cvec3d& camerapos);
  void DrawConstraints();
  void DrawDragArc();
  void DrawResultArc();

  // data:
  double radius;

  // position  the world coord system
  cvec3d center;
  
  //  combined transformation matrix
  HMatrix combined;

  Quat  qNow, qDown, qDrag;
  // current picking ray and the picking ray when drag was started
  cvec3d rayStartNow, rayDirNow,rayStartDown, rayDirDown;
  cvec3d vFrom, vTo, vrFrom, vrTo;
  HMatrix mNow, mDown;
  bool showResult, dragging;
  cvec3d sets[NSets][MAX_SET_SIZE];
  int setSizes[NSets];
  AxisSet axisSet;
  int axisIndex;

  cvec3f outlineColor;
  cvec3f inactiveColor;
  cvec3f activeColor;
  cvec3f dragColor;
  cvec3f resultColor;

};

#endif /* __ARCBALL_H__ */







