// -*- Mode: c++ -*-
// $Id: controls.cpp,v 1.15 2000/08/09 16:44:59 jianbo Exp $
// $Source: /tools/cvs-repos/illustrate/src/controls.cpp,v $

#include <stdio.h>
#include <GL/glut.h>
#include <pu.h>

#include "compat.h"
#include "options.h"
#include "silviewer.h"
#include "silhouette.h"
#include "controls.h"

SilViewer* Controls::silviewer = 0;

// window parameters
int  Controls::_winId;
int  Controls::_width = 100;
int  Controls::_height = 300;
char Controls::_title[80];

// depth button label
char    Controls::_depthInfo[10];
puText* Controls::_depthLabel = 0;

// mesh control buttons
puButton * Controls::btndrawRays = 0;
puButton * Controls::btndrawBoundary = 0;
puButton * Controls::btndrawSilhouetteEdges = 0;
puButton * Controls::btndrawMeshGrid = 0;
puButton * Controls::btndrawFaceNormal = 0;
puButton * Controls::btndrawWireframe = 0;
puButton * Controls::btnlighting = 0;

// constants
const int Controls::buttonWidth = 150;
const int Controls::buttonHeight = 30;
const int Controls::inputWidth = 150;
const int Controls::inputHeight = 30;
const int Controls::radioButtonWidth = 16;
const int Controls::radioButtonHeight = 16;
const int Controls::radioButtonTextOffset = 16;
const int Controls::sliderWidth = 100;
const int Controls::sliderHeight = 20;
const int Controls::textLabelHeight = 25;
const int Controls::buttonOffset = 10;

void Controls::createWindow(char *title) {
  if (title) strcpy(_title, title);
  else strcpy(_title, "Controls");

  glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
  _winId = glutCreateWindow( _title );

  puInit();
  
  createElements( _width, _height);
  glutReshapeWindow( _width, _height );

  glutDisplayFunc(display);
  glutMouseFunc(mouse);
  glutMotionFunc(motion);
  glutKeyboardFunc(keyboard);
}

void Controls::redisplay() {
  glutSetWindow(_winId);
  glutPostRedisplay();
}

void Controls::display( void ) {
  glClearColor( 0.5, 0.5, 0.5, 0.0 );
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  puDisplay();
  glutSwapBuffers();
}

void Controls::mouse( int button, int state, int x, int y ) {
  puMouse(button,state,x,y);
  redisplay();
}

void Controls::motion( int x, int y ) {
  puMouse(x,y);
  redisplay();
}

void Controls::keyboard( unsigned char key, int x, int y ) {
  puKeyboard(key,PU_DOWN);
  redisplay();
}

void Controls::createElements( int& width, int& height ) {
  width = buttonOffset;
  height = 10;
  puOneShot *btnquit = createButton("quit", buttonOffset, height);
  btnquit->setCallback( quit );
  width = max(width, buttonOffset + btnquit->getBBox()->max[0]);
  height += buttonHeight + 5;

  puOneShot *btninputCamera = createButton("input camera", buttonOffset, height);
  btninputCamera->setCallback( inputCamera );
  width = max(width, buttonOffset + btninputCamera->getBBox()->max[0]);
  height += buttonHeight + 5;

  puOneShot *btnoutputCamera = createButton("output camera", buttonOffset, height);
  btnoutputCamera->setCallback( outputCamera );
  width = max(width, buttonOffset + btnoutputCamera->getBBox()->max[0]);
  height += buttonHeight + 5;
  
  puOneShot *btnsilhouette = createButton("silhouette", buttonOffset, height);
  btnsilhouette->setCallback( recomputeSilhouettes );
  width = max(width, buttonOffset + btnsilhouette->getBBox()->max[0]);
  height += buttonHeight + 5;

  puOneShot *btnsubdivide = createButton("subdivide", buttonOffset, height);
  btnsubdivide->setCallback( subdivide );
  if (Options::depth >= Options::maxDepth) {
    btnsubdivide->setColor(PUCOL_LEGEND, 0.25, 0.25, 0.25);
    btnsubdivide->greyOut();
  }
  width = max(width, buttonOffset + btnsubdivide->getBBox()->max[0]);
  height += buttonHeight;

  sprintf(_depthInfo, "depth %d\0", Options::depth);
  _depthLabel = createLabel(_depthInfo, buttonOffset, height);
  width = max(width, buttonOffset + _depthLabel->getBBox()->max[0]);
  height += textLabelHeight;
  
  btndrawRays = createRadioButton("show rays", buttonOffset*2, height);
  btndrawRays->setCallback( drawRays );
  width = max(width, buttonOffset + btndrawRays->getBBox()->max[0]);
  height += radioButtonHeight + 2;
  
  btndrawBoundary = createRadioButton("mark boundary", buttonOffset*2, height);
  btndrawBoundary->setCallback( drawBoundary );
  width = max(width, buttonOffset + btndrawBoundary->getBBox()->max[0]);
  height += radioButtonHeight + 2;

  btndrawSilhouetteEdges = createRadioButton("mark silhouette", buttonOffset*2, height);
  btndrawSilhouetteEdges->setCallback( drawSilhouetteEdges );
  btndrawSilhouetteEdges->setValue( Options::drawSilhouetteEdges );
  width = max(width, buttonOffset + btndrawSilhouetteEdges->getBBox()->max[0]);
  height += radioButtonHeight + 2;

  btndrawMeshGrid = createRadioButton("show grid", buttonOffset*2, height);
  btndrawMeshGrid->setCallback( drawMeshGrid );
  btndrawMeshGrid->setValue( Options::drawMeshGrid );
  width = max(width, buttonOffset + btndrawMeshGrid->getBBox()->max[0]);
  height += radioButtonHeight + 2;

  btndrawFaceNormal = createRadioButton("face normal", buttonOffset*2, height);
  btndrawFaceNormal->setCallback( drawFaceNormal );
  btndrawFaceNormal->setValue( Options::drawFaceNormal );
  width = max(width, buttonOffset + btndrawFaceNormal->getBBox()->max[0]);
  height += radioButtonHeight + 2;

  btndrawWireframe = createRadioButton("wireframe", buttonOffset*2, height);
  btndrawWireframe->setCallback( drawWireframe );
  btndrawWireframe->setValue( Options::drawWireframe );
  width = max(width, buttonOffset + btndrawWireframe->getBBox()->max[0]);
  height += radioButtonHeight + 2;

  btnlighting = createRadioButton("lighting", buttonOffset*2, height);
  btnlighting->setCallback( lighting );
  btnlighting->setValue( Options::lighting );
  width = max(width, buttonOffset + btnlighting->getBBox()->max[0]);
  height += radioButtonHeight + 2;

  puButton *btnviewMesh = createRadioButton("original mesh", buttonOffset, height);
  btnviewMesh->setCallback( viewMesh );
  setMeshButtons( false );
  width = max(width, buttonOffset + btnviewMesh->getBBox()->max[0]);
  height += textLabelHeight;

  puButton *btnshowDualMesh = createRadioButton("dual mesh", buttonOffset, height);
  btnshowDualMesh->setCallback( showDualMesh );
  width = max(width, buttonOffset + btnshowDualMesh->getBBox()->max[0]);
  height += radioButtonHeight + 2;
  
  puButton *btndrawSilGrid = createRadioButton("show grid", buttonOffset, height);
  btndrawSilGrid->setCallback( drawSilGrid );
  btndrawSilGrid->setValue( Options::drawSilGrid );
  width = max(width, buttonOffset + btndrawSilGrid->getBBox()->max[0]);
  height += radioButtonHeight + 2;
  
  puButton *btndrawQI = createRadioButton("show QI", buttonOffset, height);
  btndrawQI->setCallback( drawQI );
  btndrawQI->setValue( Options::drawQI );
  width = max(width, buttonOffset + btndrawQI->getBBox()->max[0]);
  height += radioButtonHeight + 2;
  
  puButton *btndrawIntersections = createRadioButton("intersection", buttonOffset, height);
  btndrawIntersections->setCallback( drawIntersections );
  btndrawIntersections->setValue( Options::drawIntersections );
  width = max(width, buttonOffset + btndrawIntersections->getBBox()->max[0]);
  height += radioButtonHeight + 2;

  puButton *btndistinctChainColors = createRadioButton("distinct color", buttonOffset, height);
  btndistinctChainColors->setCallback( distinctChainColors );
  btndistinctChainColors->setValue( Options::distinctChainColors );
  width = max(width, buttonOffset + btndistinctChainColors->getBBox()->max[0]);
  height += radioButtonHeight + 2;
  
  puButton *btndrawHiddenCurves = createRadioButton("hidden curve", buttonOffset, height);
  btndrawHiddenCurves->setCallback( drawHiddenCurves );
  btndrawHiddenCurves->setValue( Options::drawHiddenCurves );
  width = max(width, buttonOffset + btndrawHiddenCurves->getBBox()->max[0]);
  height += radioButtonHeight + 2;
  
  puButton *btnvisibility = createRadioButton("visibility", buttonOffset, height);
  btnvisibility->setCallback( visibility );
  btnvisibility->setValue( Options::visibility );
  width = max(width, buttonOffset + btnvisibility->getBBox()->max[0]);
  height += radioButtonHeight + 2;
  
  puButton *btntakeBoundary = createRadioButton("boundary", buttonOffset, height);
  btntakeBoundary->setCallback( takeBoundary );
  btntakeBoundary->setValue( Options::takeBoundary );
  width = max(width, buttonOffset + btntakeBoundary->getBBox()->max[0]);
  height += radioButtonHeight + 2;

  puText *txtIllustration = createLabel("illustration:", buttonOffset, height);
  height += textLabelHeight;
}

puText* Controls::createLabel(char *text, int x, int y) {
  puText *t = new puText(x, y);
  t->setLabel(text);
  t->setLabelFont(PUFONT_8_BY_13);
  t->setLegendFont(PUFONT_8_BY_13);
  return t;
}

puOneShot* Controls::createButton(char *text, int x, int y) {
  puOneShot *b = new puOneShot(x,y,x+buttonWidth, y+buttonHeight);
  b->setLegend( text );
  b->setLegendFont( PUFONT_8_BY_13 );
  b->setColor( PUCOL_LEGEND, 0, 0, 0 );
  b->setStyle( PUSTYLE_SMALL_SHADED );
  return b;
}

puButton* Controls::createRadioButton(char *text, int x, int y) {
  puButton *b = new puButton(x, y, x+radioButtonWidth, y+radioButtonHeight);
  b->setLabel( text );
  b->setLabelFont( PUFONT_8_BY_13 );
  b->setColor( PUCOL_LABEL, 0, 0, 0 );
  b->setStyle( PUSTYLE_RADIO );
  return b;
}

void Controls::quit(puObject *b) {
  exit(0);
}

void Controls::outputCamera(puObject *b) {
  if (silviewer) {
    ofstream c(Options::camerafile);
    if ( !c ) {
      cerr << "can't create file : " << Options::camerafile << endl;
    } else {
      silviewer->outputCamera(c);
      cerr << "camera wrote to file : " << Options::camerafile << endl;
    }
  }
}

void Controls::inputCamera(puObject *b) {
  if (silviewer) {
    ifstream c(Options::camerafile);
    if ( !c ) {
      cerr << "can't find file : " << Options::camerafile << endl;
    } else {
      silviewer->inputCamera(c);
      cerr << "camera read from file : " << Options::camerafile << endl;
    }
  }
}

void Controls::recomputeSilhouettes(puObject *b) {
  if (silviewer) {
    silviewer->useMeshCamera();
    silviewer->recomputeSilhouettes();
    silviewer->redisplay();
    silviewer->redisplayMesh();
    silviewer->redisplayDualMesh();
  }
  redisplay();
}

void Controls::subdivide(puObject *b) {
  Options::depth++;
  if (silviewer)
    if (silviewer->mesh()) {
      silviewer->subdivide(Options::depth);
      silviewer->initSilhouettes();
      silviewer->redisplay();
      silviewer->clearMeshDisplayList();
      silviewer->redisplayMesh();
      silviewer->redisplayDualMesh();
    }

  sprintf(_depthInfo, "depth %d\0", Options::depth);
  _depthLabel->setLabel(_depthInfo);
  if (Options::depth >= Options::maxDepth) {
    b->greyOut();
    b->setColor(PUCOL_LEGEND, 0.25, 0.25, 0.25);
  }
  redisplay();
}

void Controls::takeBoundary(puObject *b) {
  Options::takeBoundary = b->getValue();
  redisplay();
  if (silviewer && OcTree::boundary.size() > 0) {
    silviewer->recomputeSilhouettes();
    silviewer->redisplay();
    silviewer->redisplayMesh();
    silviewer->redisplayDualMesh();
  }
}

void Controls::visibility(puObject *b) {
  Options::visibility = b->getValue();
  redisplay();
  if (silviewer) silviewer->redisplay();
}

void Controls::drawHiddenCurves(puObject *b) {
  Options::drawHiddenCurves = b->getValue();
  redisplay();
  if (silviewer) silviewer->redisplay();
}

void Controls::distinctChainColors(puObject *b) {
  Options::distinctChainColors = b->getValue();
  redisplay();
  if (silviewer) silviewer->redisplay();
}

void Controls::drawIntersections(puObject *b) {
  Options::drawIntersections = b->getValue();
  redisplay();
  if (silviewer) silviewer->redisplay();
}

void Controls::drawQI(puObject *b) {
  Options::drawQI = b->getValue();
  redisplay();
  if (silviewer) silviewer->redisplay();
}

void Controls::drawSilGrid(puObject *b) {
  Options::drawSilGrid = b->getValue();
  redisplay();
  if (silviewer) silviewer->redisplay();
}

void Controls::showDualMesh(puObject *b) {
  if (silviewer) silviewer->viewDualMesh(b->getValue() != 0);
  redisplay();
}

void Controls::setMeshButtons(bool v) {
  if (v) {
    btndrawRays->activate();
    btndrawRays->setColor(PUCOL_LABEL, 0, 0, 0);
    btndrawBoundary->activate();
    btndrawBoundary->setColor(PUCOL_LABEL, 0, 0, 0);
    btndrawSilhouetteEdges->activate();
    btndrawSilhouetteEdges->setColor(PUCOL_LABEL, 0, 0, 0);
    btndrawMeshGrid->activate();
    btndrawMeshGrid->setColor(PUCOL_LABEL, 0, 0, 0);
    btndrawFaceNormal->activate();
    btndrawFaceNormal->setColor(PUCOL_LABEL, 0, 0, 0);
    btndrawWireframe->activate();
    btndrawWireframe->setColor(PUCOL_LABEL, 0, 0, 0);
    btnlighting->activate();
    btnlighting->setColor(PUCOL_LABEL, 0, 0, 0);
  } else {
    btndrawRays->greyOut();
    btndrawRays->setColor(PUCOL_LABEL, 0.25, 0.25, 0.25);
    btndrawBoundary->greyOut();
    btndrawBoundary->setColor(PUCOL_LABEL, 0.25, 0.25, 0.25);
    btndrawSilhouetteEdges->greyOut();
    btndrawSilhouetteEdges->setColor(PUCOL_LABEL, 0.25, 0.25, 0.25);
    btndrawMeshGrid->greyOut();
    btndrawMeshGrid->setColor(PUCOL_LABEL, 0.25, 0.25, 0.25);
    btndrawFaceNormal->greyOut();
    btndrawFaceNormal->setColor(PUCOL_LABEL, 0.25, 0.25, 0.25);
    btndrawWireframe->greyOut();
    btndrawWireframe->setColor(PUCOL_LABEL, 0.25, 0.25, 0.25);
    btnlighting->greyOut();
    btnlighting->setColor(PUCOL_LABEL, 0.25, 0.25, 0.25);
  }
}

void Controls::viewMesh(puObject *b) {
  bool v = b->getValue() != 0;
  setMeshButtons(v);
  if (silviewer) silviewer->viewMesh(v);
  redisplay();
}

void Controls::drawMeshGrid(puObject *b) {
  Options::drawMeshGrid = b->getValue();
  redisplay();
  if (silviewer) silviewer->redisplayMesh();
}

void Controls::drawRays(puObject *b) {
  Options::drawRays = b->getValue();
  redisplay();
  if (silviewer) silviewer->redisplayMesh();
}

void Controls::drawBoundary(puObject *b) {
  Options::drawBoundary = b->getValue();
  redisplay();
  if (silviewer && OcTree::boundary.size() > 0)
    silviewer->redisplayMesh();
}

void Controls::drawSilhouetteEdges(puObject *b) {
  Options::drawSilhouetteEdges = b->getValue();
  redisplay();
  if (silviewer) silviewer->redisplayMesh();
}

void Controls::lighting(puObject *b) {
  Options::lighting = b->getValue();
  redisplay();
  if (silviewer) silviewer->redisplayMesh();
}

void Controls::drawFaceNormal(puObject *b) {
  Options::drawFaceNormal = b->getValue();
  redisplay();
  if (silviewer) {
    silviewer->clearMeshDisplayList();
    silviewer->redisplayMesh();
  }
}

void Controls::drawWireframe(puObject *b) {
  Options::drawWireframe = b->getValue();
  redisplay();
  if (silviewer) {
    silviewer->clearMeshDisplayList();
    silviewer->redisplayMesh();
  }
}
