// -*- Mode: c++ -*-
// $Id: dualviewer.cpp,v 1.14 2000/08/08 21:28:16 jianbo Exp $
// $Source: /tools/cvs-repos/illustrate/src/dualviewer.cpp,v $

#if defined(_WIN32) || defined(__sgi)
#include <float.h>
#endif
#include <GL/glut.h>
#include <stdio.h>
#include "dualmesh.h"
#include "dualviewer.h"

DualViewer::DualViewer(int width, int height)
  : Viewer("Dual Mesh #0", width, height),
    meshNumber(0), buttonDown(false), _dualmesh(0),
    _camera(), _arcball() {
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  int vp[] = {0, 0, width, height};
  _camera.setViewport(vp);
  double w = 2.0 / tan(M_PI/18.0);
  _camera.translate(cvec3d(0, 0, -w));
  _camera.setPerspectiveParams(20, 0.1, 3.0 + w);
  _camera.computeModelview();
  _camera.computeProjection();
  _camera.loadMatrices();

  _arcball.SetRadius(2.0);
  _arcball.Update();
}

void DualViewer::setCameraPosition(const cvec3d& pos) {
  _planeNormal[0] = cvec3d(pos.y(), pos.z(), 1), _planeD[0] = pos.x();
  _planeNormal[1] = cvec3d(pos.z(), 1, pos.x()), _planeD[1] = pos.y();
  _planeNormal[2] = cvec3d(1, pos.x(), pos.y()), _planeD[2] = pos.z();
  _planeNormal[3] = pos, _planeD[3] = 1;
  _planeNormal[4] = cvec3d(pos.y(), pos.z(), 1), _planeD[4] = -pos.x();
  _planeNormal[5] = cvec3d(pos.z(), 1, pos.x()), _planeD[5] = -pos.y();
  _planeNormal[6] = cvec3d(1, pos.x(), pos.y()), _planeD[6] = -pos.z();
  _planeNormal[7] = pos, _planeD[7] = -1;
}

void DualViewer::display() {
  glPushMatrix();

  glMultMatrixd(_arcball.Rotation());

  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  glLineWidth(3);
  glColor3ub(0, 0, 0);
  glBegin(GL_QUAD_STRIP);
  glVertex3i(1, 1, 1);
  glVertex3i(-1, 1, 1);
  glVertex3i(1, -1, 1);
  glVertex3i(-1, -1, 1);
  glVertex3i(1, -1, -1);
  glVertex3i(-1, -1, -1);
  glVertex3i(1, 1, -1);
  glVertex3i(-1, 1, -1);
  glVertex3i(1, 1, 1);
  glVertex3i(-1, 1, 1);
  glEnd();
  glLineWidth(1);

  glColor3ub(255, 0, 0);
  render();

  glPopMatrix();
}

void DualViewer::reshape(int w, int h) {
  if ((w == 0) || (h == 0) || ((w == _width) && (h == _height))) return;
  _width = w;
  _height = h;
  glViewport(0, 0, w, h);

  int vp[] = {0, 0, w, h};
  _camera.setViewport(vp);
  _camera.computeModelview();
  _camera.computeProjection();
  _camera.loadMatrices();
  glutPostRedisplay();
}

void DualViewer::mouse(int button, int state, int x, int y) {
  buttonDown = false;
  y = _height - y;
  if (button == GLUT_LEFT_BUTTON) {
    // begin rotation
    if (state == GLUT_DOWN) {
      buttonDown = true;
      cvec3d worldMousePos = _camera.unproject(cvec3d(x, y, 0));
      _arcball.PickingRay(_camera.viewWorldPosition(),
			  (worldMousePos - _camera.viewWorldPosition()).dir());
      _arcball.BeginDrag();
      _arcball.Update();
    } else { // finish rotation
      _camera.model() = _camera.model() * _arcball.Rotation();
      _camera.computeModelview();
      _camera.loadMatrices();
      _arcball.SetRotation(HMatrix::Identity());
    }
    glutPostRedisplay();
  } else if (button == GLUT_RIGHT_BUTTON && state == GLUT_UP) {
    meshNumber = (meshNumber+1) % 8;
    char title[32];
    sprintf(title, "Dual Mesh #%d", meshNumber);
    glutSetWindowTitle(title);
    glutPostRedisplay();
  } 
}

void DualViewer::motion(int x, int y) {
  if (buttonDown) {
    y = _height - y;
    cvec3d worldMousePos = _camera.unproject(cvec3d(x, y, 0));
    _arcball.PickingRay(_camera.viewWorldPosition(),
			(worldMousePos - _camera.viewWorldPosition()).dir());
    _arcball.Update();
    glutPostRedisplay();
  }
}

void DualViewer::setClipping() {
  GLdouble planeEq[][4] = {{-1, 0, 0, 1}, {1, 0, 0, 1},
                           {0, -1, 0, 1}, {0, 1, 0, 1},
                           {0, 0, -1, 1}, {0, 0, 1, 1}};
  glClipPlane(GL_CLIP_PLANE0, planeEq[0]);
  glClipPlane(GL_CLIP_PLANE1, planeEq[1]);
  glClipPlane(GL_CLIP_PLANE2, planeEq[2]);
  glClipPlane(GL_CLIP_PLANE3, planeEq[3]);
  glClipPlane(GL_CLIP_PLANE4, planeEq[4]);
  glClipPlane(GL_CLIP_PLANE5, planeEq[5]);

  glEnable(GL_CLIP_PLANE0);
  glEnable(GL_CLIP_PLANE1);
  glEnable(GL_CLIP_PLANE2);
  glEnable(GL_CLIP_PLANE3);
  glEnable(GL_CLIP_PLANE4);
  glEnable(GL_CLIP_PLANE5);
}

void DualViewer::disableClipping() {
  glDisable(GL_CLIP_PLANE0);
  glDisable(GL_CLIP_PLANE1);
  glDisable(GL_CLIP_PLANE2);
  glDisable(GL_CLIP_PLANE3);
  glDisable(GL_CLIP_PLANE4);
  glDisable(GL_CLIP_PLANE5);
}

void DualViewer::render() {
  if (_dualmesh == 0) return;

  DualEdge *e;

  // dual mesh
  glLineWidth(2);
  glBegin(GL_LINES);
  int dsz = _dualmesh[meshNumber].size();
  for (int i=0; i<dsz; i++) {
    e = _dualmesh[meshNumber].edges()[i];
//      glNormal3dv(1.0/3 * (e->tri(0)->vert(0)->limitPos() +
//  			  e->tri(0)->vert(1)->limitPos() +
//  			  e->tri(0)->vert(2)->limitPos()));
    glVertex3dv(e->v(0));
//      glNormal3dv(1.0/3 * (e->tri(1)->vert(0)->limitPos() +
//  			  e->tri(1)->vert(1)->limitPos() +
//  			  e->tri(1)->vert(2)->limitPos()));
    glVertex3dv(e->v(1));
  }
  glEnd();

  // the camera plane
  // normal
  cvec3d n = _planeNormal[meshNumber];
  // project orig onto the plane
  cvec3d p = - _planeD[meshNumber]/n.dot() * n;
  // find two vectors in the plane
  cvec3d q1, q2;
  n.normalize();
  if (fabs(n.x()) > 0.5) {
    q1 = cvec3d(n.y(), - n.x(), 0).dir() * 3;
    q2 = cvec3d(n.x()*n.z(), n.y()*n.z(),
		- n.x()*n.x() - n.y()*n.y()).dir() * 3;
  } else {
    q1 = cvec3d(0, n.z(), -n.y()).dir() * 3;
    q2 = cvec3d(- n.y()*n.y() - n.z()*n.z(),
		n.x()*n.y(), n.x()*n.z()).dir() * 3;
  }

  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glEnable(GL_BLEND);
  setClipping();

  glColor4ub(64, 64, 192, 160);
  glBegin(GL_QUADS);
  glVertex3dv(p + q1);
  glVertex3dv(p + q2);
  glVertex3dv(p - q1);
  glVertex3dv(p - q2);
  glEnd();

  disableClipping();
  glDisable(GL_BLEND);
}
