// -*- mode: c++ -*-
// $Id: grid3d.h,v 1.6 2000/08/08 21:28:16 jianbo Exp $
// $Source: /tools/cvs-repos/illustrate/src/grid3d.h,v $

#ifndef __GRID3D_H__
#define __GRID3D_H__

#include <vector>
#include "subtri.h"
#include "tagmesh.h"

// base mesh type
typedef TagMeshTp<Tri> TMesh;

// ray class
class Ray {
public:
  Ray(const cvec3d& s, const cvec3d& d) : start(s), end(d), p(-1) { };

  // start position and end position
  cvec3d start, end;

  // if intersect with mesh, the intersection will be (1-p)*start + p*end
  // else p = -1
  double p;
};

class GridTri {
public:
  GridTri(Tri *tp) : t(tp),
    n(tp->normal()) {
    if (t->orientation()) {
      en1 = (v(0)-v(1)).cross(n).dir();
      en2 = (v(1)-v(2)).cross(n).dir();
      en3 = (v(2)-v(0)).cross(n).dir();
    } else {
      en1 = (v(1)-v(0)).cross(n).dir();
      en2 = (v(2)-v(1)).cross(n).dir();
      en3 = (v(0)-v(2)).cross(n).dir();
    }
  };

  const cvec3d& v(VnoType i) const
    { return t->vert(i)->limitPos(); };

  // original triangle
  Tri *t;
  // face normal and edge normal (point inside of the triangle)
  cvec3d n, en1, en2, en3;
};

class Grid3D {
public:
  Grid3D() : _size(0), _xstep(0), _ystep(0), _zstep(0),
	     _min(0), _max(0), _grid(0) { };
  Grid3D(int s) : _size(0), _xstep(0), _ystep(0), _zstep(0),
		  _min(0), _max(0), _grid(0)
  { if (s>0) createGrid(s); };
  ~Grid3D();

  // member accessors
  int size() const { return _size; };
  double xstep() const { return _xstep; };
  double ystep() const { return _ystep; };
  double zstep() const { return _zstep; };
  cvec3d& minp() { return _min; };
  cvec3d& maxp() { return _max; };
  const cvec3d& minp() const { return _min; };
  const cvec3d& maxp() const { return _max; };
  vector<Ray>& rays() { return _rays; };
  // clear all rays
  void clearRays() { _rays.clear(); };

  // place a triangle mesh in the grid
  void placeIntoGrid(TMesh *mesh);
  // intersect a ray with the mesh
  bool intersectRay(const cvec3d& start, const cvec3d& end);

private:
  // grid size
  int _size;
  // cell length in x, y, z directions
  double _xstep, _ystep, _zstep;
  // bounding box of the mesh
  cvec3d _min, _max;
  // rays tested for intersection
  vector<Ray> _rays;
  // cells
  vector<GridTri*> *** _grid;

  // allocate proper space for a grid of size s, free old
  void createGrid(int s);
  // place a triangle into intesected cells
  void placeTriangle(Tri *t);
};

#endif
