// -*- Mode: c++ -*-
// $Id: options.cpp,v 1.13 2000/08/09 16:45:00 jianbo Exp $
// $Source: /tools/cvs-repos/illustrate/src/options.cpp,v $

#include <stdlib.h>
#include <string.h>
#include "compat.h"
#include "options.h"

const double Options::EPSILON = 1e-10;

// mesh file name
char* Options::inputfile = 0;

// camera file name
char* Options::camerafile = 0;

// subdivision depth
int Options::depth = 0;
int Options::maxDepth = 4;

// window size
int Options::width = 600;
int Options::height = 450;
int Options::dualWinSize = 300;

// mesh viewer options
int Options::drawSilhouetteEdges = 0;
int Options::drawBoundary = 0;
int Options::drawWireframe = 0;
int Options::lighting = 1;
int Options::drawFaceNormal = 0;
int Options::drawRays = 0;
int Options::drawMeshGrid = 0;

// illustration options
int Options::takeBoundary = 1;
int Options::visibility = 1;
int Options::drawHiddenCurves = 0;
int Options::distinctChainColors = 0;
int Options::drawIntersections = 0;
int Options::drawQI = 0;
int Options::drawSilGrid = 0;

// command line parser
void Options::readCommandLine(int argc, char ** argv) {
  int k = 1;
  while (k < argc) {
    if (strcmp(argv[k], "-depth") == 0) {
      if (k+1 >= argc) {
	cerr<< "Please give the depth with -depth option." << endl;
	exit(0);
      }
      int d = atoi(argv[k+1]);
      if (d < 0)
	cerr<< "Warning: Subdivision depth can't be negative. Ignored."
	    << endl;
      else Options::depth = d;
      k += 2;
    } else {
      if (inputfile)
	cerr<< "Warning: Multiple input files. " << argv[k] << " ignored."
	    << endl;
      else inputfile = strdup(argv[k]);
      k ++;
    }
  }
  if (!inputfile) {
    cerr<< "Usage: " << argv[0] << " inputfile [-depth d]" << endl;
    exit(0);
  }

  char *tb1 = strrchr(inputfile, '/');
  char *tb2 = strrchr(inputfile, '\\');
  char *tb = (tb1 > tb2) ? tb1+1 : (tb2 ? tb2+1 : inputfile);

  camerafile = new char[8 + strlen(tb)];
  char *ti = strrchr(tb, '.');

  if (ti) {
    strncpy(camerafile, tb, ti - tb);
    camerafile[ti - tb] = '\0';
  }
  else
    strcpy(camerafile, tb);
  strcat(camerafile, ".camera");
}
